#!/usr/bin/env python
""" Test script for slurm remote interface """

import sys
import argparse
from biobb_remote.slurm import Slurm

ARGPARSER = argparse.ArgumentParser(
    description='Test Slurm script for biobb_remote'
)
ARGPARSER.add_argument(
    dest='command',
    help='Remote command',
    choices=['submit', 'queue', 'cancel', 'status', 'get_data', 'put_data', 'logs', 'get_file']
)
ARGPARSER.add_argument(
    '--keys_path',
    dest='keys_path',
    help='Credentials file path',
    required=True
)
ARGPARSER.add_argument(
    '--script',
    dest='local_run_script',
    help='Path to local script'
)
ARGPARSER.add_argument(
    '--local_data',
    dest='local_data_path',
    help='Local data bundle'
)
ARGPARSER.add_argument(
    '--remote',
    dest='remote_path',
    help='Remote working dir'
)
ARGPARSER.add_argument(
    '--queue_settings',
    dest='queue_settings',
    help='Predefined queue settings'
)
ARGPARSER.add_argument(
    '--modules',
    dest='modules',
    help='Software modules to load'
)
ARGPARSER.add_argument(
    '--task_data_file',
    dest='task_file_path',
    help='Store for task data'
)

ARGPARSER.add_argument(
    '--overwrite',
    dest='overwrite',
    action='store_true',
    help='Overwrite data in output local directory'
)

ARGPARSER.add_argument(
    '--task_file_type',
    dest='task_file_type',
    default='json',
    help='Format for task data file (json, pickle). Default:json'
)

ARGPARSER.add_argument(
    '--poll',
    dest='polling_int',
    default=0,
    type=int,
    help='Polling interval (min), 0: No polling (default)'
)

ARGPARSER.add_argument(
    '--remote_file',
    dest='remote_file',
    help='Remote file name to download (get_file)'
)

class Slurm_test():
    def __init__(self, args):
        self.args = args

    def launch(self):
        slurm_task = Slurm()
        slurm_task.set_credentials(self.args.keys_path)

        if self.args.command not in ('queue', 'submit'):
            try:
                slurm_task.load_data_from_file(self.args.task_file_path)
                print("Task data loaded from", self.args.task_file_path)
            except IOError:
                print("Task data not loaded")

        if self.args.command == 'submit':
            slurm_task.set_local_data_bundle(self.args.local_data_path)
            if 'input_data_loaded' not in slurm_task.task_data:
                slurm_task.send_input_data(self.args.remote_path)
            slurm_task.submit(self.args.queue_settings, self.args.modules, self.args.local_run_script)
            print("job id", slurm_task.task_data['remote_job_id'])

        elif self.args.command == 'cancel':
            slurm_task.cancel(remove_data=True)
            print('job ' + slurm_task.task_data['remote_job_id'] + '  cancelled')

        elif self.args.command == 'queue':
            stdout, stderr = slurm_task.check_queue()
            print(''.join(stdout))
            print(''.join(stderr), file=sys.stderr)

        elif self.args.command == 'status':
            slurm_task.check_job()

        elif self.args.command == 'get_data':
            slurm_task.get_output_data(self.args.local_data_path, False)

        elif self.args.command == 'put_data':
            slurm_task.set_local_data_bundle(self.args.local_data_path)
            slurm_task.send_input_data(self.args.remote_path)

        elif self.args.command == 'logs':
            stdout, stderr = slurm_task.get_logs()
            print("Job Output log")
            print(stdout)
            print("Jog Error log")
            print(stderr)

        elif self.args.command == 'get_file':
            print(slurm_task.get_remote_file(self.args.remote_file))

        else:
            sys.exit("test_slurm: error: unknown command " + self.args.command)

        if slurm_task.modified:
            if not self.args.task_file_path:
                self.args.task_file_path = slurm_task.id + ".task"
            try:
                slurm_task.save(self.args.task_file_path)
                print("Task data saved on", self.args.task_file_path)
            except IOError as e:
                sys.exit(e)

def main():
    args = ARGPARSER.parse_args()
    stest = Slurm_test(args).launch()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
""" Module to handle ssh credentials (userid, host, private_keys) """

__author__ = "gelpi"
__date__ = "$08-March-2019 17:32:38$"

import sys
import argparse
import os
from paramiko import SSHClient, AutoAddPolicy, AuthenticationException
from biobb_remote.ssh_credentials import SSHCredentials

ARGPARSE = argparse.ArgumentParser(
    description='Credentials manager for biobb_remote'
)
ARGPARSE.add_argument(
    dest='operation',
    help='Operation: create|get_pubkey',
    choices=['create', 'get_pubkey', 'get_private', 'host_install', 'host_remove', 'host_check']
)
ARGPARSE.add_argument(
    '--user',
    dest='userid',
    help='User id'
)
ARGPARSE.add_argument(
    '--host',
    dest='hostname',
    help='Host name'
)
ARGPARSE.add_argument(
    '--pubkey_path',
    dest='pubkey_path',
    help='Public key file path'
)
ARGPARSE.add_argument(
    '--nbits',
    dest='nbits',
    type=int,
    default=2048,
    help='Number of key bits'
)
ARGPARSE.add_argument(
    '--keys_path',
    dest='keys_path',
    help='Credentials file path',
    required=True
)
ARGPARSE.add_argument(
    '--privkey_path',
    dest='privkey_path',
    help='Private key file path'
)
ARGPARSE.add_argument(
   '-v',
   dest="verbose",
   action="store_true",
   help='Output extra information'
)


class Credentials():
    """ Class to wrap credentials management following biobb_template"""
    def __init__(self, line_args):
        self.args = line_args

    def launch(self):
        """ Launch execution following biobb_template"""
        if self.args.operation == 'create':
            credentials = SSHCredentials(
                host=self.args.hostname,
                userid=self.args.userid,
                generate_key=False
            )
            credentials.generate_key(self.args.nbits)
            credentials.save(
                output_path=self.args.keys_path,
                public_key_path=self.args.pubkey_path,
                private_key_path=self.args.privkey_path
            )
            if self.args.verbose:
                print("Credentials stored in", self.args.keys_path)
            if self.args.pubkey_path is None:
                print("Public key, add to authorized_keys on remote host")
                print(credentials.get_public_key())
        else:
            credentials = SSHCredentials()
            credentials.load_from_file(self.args.keys_path)

            if self.args.operation == 'get_pubkey':
                print(credentials.get_public_key())

            elif self.args.operation == 'get_private':
                print(credentials.get_private_key())

            elif self.args.operation in ('host_install', 'host_remove', 'host_check'):
                host_str = '{}@{}'.format(credentials.userid, credentials.host)

                if self.args.operation == 'host_check':
                    print('Biobb public key {} at {}'.format(
                        'found' if credentials.check_host_auth() else 'not found',
                        host_str
                    )
                    )
                elif self.args.operation == 'host_install':
                    if not credentials.check_host_auth():
                        credentials.install_host_auth('bck')
                        if self.args.verbose:
                            print('Biobb keys installed on', host_str)
                            print("Warning: original .ssh/authorize_keys file stored as .ssh/authorized_keys.bck")

                elif self.args.operation == 'host_remove':
                    print(credentials.check_host_auth())
                    if credentials.check_host_auth():
                        credentials.remove_host_auth('biobb')
                        if self.args.verbose:
                            print('Biobb removed from ', host_str)
                            print("Warning: .ssh/authorize_keys file stored as .ssh/authorized_keys.biobb")
            else:
                sys.exit("credentials: error: unknown op")

def main():
    args = ARGPARSE.parse_args()

    if args.operation == 'create':
        if args.userid is None or args.hostname is None:
            sys.exit("ssh_command: error: Userid and hostname are required to create credentials")

    Credentials(args).launch()

if __name__ == '__main__':
    main()



#!/usr/bin/env python

""" Command-line wrapper to SCP for biobb_remote """

__author__ = "gelpi"
__date__ = "$08-March-2019  17:32:38$"

import sys
import argparse

from biobb_remote.ssh_session import SSHSession

# COMMAND LINE ARGS
ARGPARSER = argparse.ArgumentParser(
    description='SCP wapper for biobb_remote'
)
ARGPARSER.add_argument(
    dest='operation',
    help='SCP command (get|put: Scp standard,'\
        + 'create: creates text file on remote,'\
        + 'file: prints text file on remote)',
    choices=['get', 'put', 'create', 'file', 'listdir']
)
ARGPARSER.add_argument(
    '--keys_path',
    dest='keys_path',
    help='Credentials file path',
    required=True
)
ARGPARSER.add_argument(
    '-i',
    dest='input_file_path',
    help='Input file path | input string'
)
ARGPARSER.add_argument(
    '-o',
    dest='output_file_path',
    help='Output file path'
)

class SCPService():
    """ Class wrapping scp_service following biobb_template"""
    def __init__(self, args):
        self.args = args

    def launch(self):
        """ Executes scp_service"""
        session = SSHSession(credentials_path=self.args.keys_path)
        print(
            self.args.operation,
            self.args.input_file_path,
            self.args.output_file_path
        )
        output_text = session.run_sftp(
            oper=self.args.operation,
            input_file_path=self.args.input_file_path,
            output_file_path=self.args.output_file_path
        )
        if self.args.operation == 'file':
            return output_text
        return ''

def main():
    args = ARGPARSER.parse_args()
    # Caching stdin
    if args.input_file_path is None:
        if args.operation != 'create':
            sys.exit("scp_service: error: input_file_path is required ")
        else:
            args.input_file_path = ''
            line = sys.stdin.readline()
            while line:
                args.input_file_path += line
                line = sys.stdin.readline()
    if args.output_file_path is None and args.operation != 'file':
        sys.exit("scp_service: error: output_file_path is required ")

    output = SCPService(args).launch()
    if output:
        print(output)

if __name__ == "__main__":
    main()



#!/usr/bin/env python
""" Command line utility for remote ssh command in biobb_remote"""
__author__ = "gelpi"
__date__ = "$08-March-2019 17:32:38$"

import sys
import argparse
from biobb_remote.ssh_session import SSHSession

ARGPARSER = argparse.ArgumentParser(
    description='SSH command wapper for biobb_remote'
)
ARGPARSER.add_argument(
    dest='command',
    help='Remote command',
    nargs='*'
)
ARGPARSER.add_argument(
    '--keys_path',
    dest='keys_path',
    help='Credentials file path',
    required=True
)

class SSHCommand():
    """ Class wrapping ssh_command following biobb_template"""
    def __init__(self, args):
        self.args = args

    def launch(self):
        """ Execute ssh command"""
        session = SSHSession(credentials_path=self.args.keys_path)
        if session:
            stdout, stderr = session.run_command(' '.join(self.args.command))
            print(''.join(stdout))
            print(''.join(stderr), file=sys.stderr)


def main():
    args = ARGPARSER.parse_args()
    SSHCommand(args).launch()

if __name__ == "__main__":
    main()
